"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var ExpressionUtils = /** @class */ (function () {
    function ExpressionUtils(_defaultSeparator) {
        if (_defaultSeparator === void 0) { _defaultSeparator = '_'; }
        this._defaultSeparator = _defaultSeparator;
    }
    ExpressionUtils.prototype.getPropertiesByLambdaExpression = function (expression) {
        var expressionMetadada = this.getExpressionByLambdaExpression(expression);
        return {
            alias: expressionMetadada.alias,
            operator: expressionMetadada.operator,
            propertiesLeft: this.getPropertiesByExpressionString(expressionMetadada.expressionLeft, expressionMetadada.alias),
            propertiesRight: this.getPropertiesByExpressionString(expressionMetadada.expressionRight, expressionMetadada.alias)
        };
    };
    ExpressionUtils.prototype.getAlias = function (startExpression) {
        // https://stackoverflow.com/a/17779833/2290538
        var resultRegex = utils_1.REGEX_BETWEEN_PARENTHESIS.exec(startExpression);
        return resultRegex && resultRegex.length > 1 ? resultRegex[1] : '';
    };
    ExpressionUtils.prototype.hasAlias = function (value, alias) {
        if (!alias) {
            return true;
        }
        return value.indexOf(alias + ".") > -1;
    };
    ExpressionUtils.prototype.hasBetweenQuotes = function (value) {
        return utils_1.REGEX_BETWEEN_QUOTES.test(value);
    };
    ExpressionUtils.prototype.hasExpression = function (value, alias) {
        return !this.hasBetweenQuotes(value) && this.hasAlias(value, alias);
    };
    ExpressionUtils.prototype.getValueByExpression = function (instance, expression) {
        return expression(instance);
    };
    ExpressionUtils.prototype.getValueByProperties = function (instance, properties) {
        var _this = this;
        var result = instance;
        properties.forEach(function (property) {
            result = _this.getValue(result, property);
        });
        return result;
    };
    ExpressionUtils.prototype.getValue = function (instance, property) {
        if (property.indexOf('.') > -1) {
            return this.getValueByProperties(instance, property.split('.'));
        }
        if (instance) {
            return instance[property];
        }
        return void 0;
    };
    ExpressionUtils.prototype.getColumnByExpression = function (expression, separator) {
        if (separator === void 0) { separator = this._defaultSeparator; }
        return this.getColumnByProperties(this.getPropertiesByExpression(expression), separator);
    };
    ExpressionUtils.prototype.getColumnByProperties = function (properties, separator) {
        if (separator === void 0) { separator = this._defaultSeparator; }
        return properties.join(separator);
    };
    ExpressionUtils.prototype.getPropertiesByExpression = function (expression) {
        var lambdaExpression = this.getExpressionMetadata(expression).body;
        var properties = this.getPropertiesByExpressionString(lambdaExpression.split(' ')[0]);
        return properties;
    };
    ExpressionUtils.prototype.getExpressionMetadata = function (expression) {
        var expressionAsString = expression.toString();
        if (expressionAsString.search('=>') !== -1) {
            return this.getExpressionMetadataES2015(expressionAsString);
        }
        else if (expressionAsString.search('return') !== -1) {
            return this.getExpressionMetadataES5(expressionAsString);
        }
        else {
            throw Error("Invalid expression: " + expressionAsString);
        }
    };
    ExpressionUtils.prototype.getExpressionMetadataES5 = function (expressionAsString) {
        var expressionMetadata = {};
        var paramsRegEx = utils_1.REGEX_BETWEEN_PARENTHESIS.exec(expressionAsString);
        expressionMetadata.params = paramsRegEx && paramsRegEx.length ? paramsRegEx[1] : '';
        expressionMetadata.body = expressionAsString.slice(expressionAsString.indexOf('return') + 7, expressionAsString.indexOf(';'));
        return expressionMetadata;
    };
    ExpressionUtils.prototype.getExpressionMetadataES2015 = function (expressionAsString) {
        var expressionMetadata = {};
        var params = expressionAsString.slice(0, expressionAsString.indexOf(' =>'));
        expressionMetadata.params = utils_1.REGEX_BETWEEN_PARENTHESIS.test(params) ? this.getAlias(params) : params;
        expressionMetadata.body = expressionAsString.slice(expressionAsString.indexOf('=>') + 3, expressionAsString.length);
        return expressionMetadata;
    };
    ExpressionUtils.prototype.getPropertiesByExpressionString = function (expression, alias) {
        if (alias === void 0) { alias = void 0; }
        if (this.hasExpression(expression, alias)) {
            var propertiesReferences = expression.split('.');
            if (propertiesReferences.length && this.hasExpression(expression, alias)) {
                propertiesReferences.shift();
            } // remove alias
            return propertiesReferences;
        }
        return [expression];
    };
    ExpressionUtils.prototype.getColumnByLambdaExpression = function (expression) {
        var propertiesMetadada = this.getPropertiesByLambdaExpression(expression);
        return {
            alias: propertiesMetadada.alias,
            columnLeft: this.getColumnByProperties(propertiesMetadada.propertiesLeft),
            columnRight: this.getColumnByProperties(propertiesMetadada.propertiesRight),
            operator: propertiesMetadada.operator
        };
    };
    ExpressionUtils.prototype.getExpressionByLambdaExpression = function (expression) {
        var lambdaExpressionMetadata = {};
        var _a = this.getExpressionMetadata(expression), params = _a.params, body = _a.body;
        lambdaExpressionMetadata.alias = params;
        var strExpression = body.split(' ');
        if (strExpression.length !== 3) {
            throw new Error("Lambda expression '" + expression.toString() + "'\n            not supported! Use simple expression with '{expressionLeft} {operador} {expressionRight}'");
        }
        lambdaExpressionMetadata.expressionLeft = strExpression[0], lambdaExpressionMetadata.operator = strExpression[1], lambdaExpressionMetadata.expressionRight = strExpression[2];
        return lambdaExpressionMetadata;
    };
    return ExpressionUtils;
}());
exports.ExpressionUtils = ExpressionUtils;
